@@ -196,7 +196,7 @@ typedef struct sizes {
 #define S_PDOUBLE   (sizeof (double *))
 #define S_PFPTR     (sizeof (int (*)()))
 #if HAVE_LONG_LONG
-#if (HOST_COMPILER == BORLANDC)
+#if (HOST_COMPILER == BORLANDC) \
         || (HOST_COMPILER == MSC && defined(_MSC_VER) && (_MSC_VER < 1300))
 #define S_LLINT     (sizeof (__int64))
 #define S_PLLINT    (sizeof (__int64 *))
@@ -1486,12 +1486,11 @@ static expr_t   eval_signed(
     case OP_OR:     v1 |= v2;           break;
     case OP_ANA:    v1 = (v1 && v2);    break;
     case OP_ORO:    v1 = (v1 || v2);    break;
-    case OP_COL:
-        /*
-         * If v1 has the "true" value, v2 has the "false" value.
-         * The top of the value stack has the test.
-         */
-        v1 = (--*valpp)->val ? v1 : v2;
+    case OP_COL:    valp--;
+        if (valp->val)
+            v1 = v1;
+        else
+            v1 = v2;
         break;
     default:
         cfatal( illeg_op, op_name, 0L, NULL);
