@@ -188,7 +188,7 @@ static char *   append_to_buffer(
     size_t      length
 )
 {
-    if (mem_buf_p->bytes_avail < length) {  /* Need to allocate more memory */
+    if (mem_buf_p->bytes_avail < length + 1) {  /* Need to allocate more memory */
         size_t size = MAX( BUF_INCR_SIZE, length);
 
         if (mem_buf_p->buffer == NULL) {            /* 1st append   */
@@ -731,9 +731,9 @@ char *  scan_quote(
     const char * const      unterm_string
                         = "Unterminated string literal%s";
     const char * const      unterm_char
-                        = "Unterminated character constant %s%.0ld%s";
+                        = "Unterminated character constant%s";
     const char * const      empty_const
-                        = "Empty character constant %s%.0ld%s";
+                        = "Empty character constant%s";
     const char *    skip;
     size_t      len;
     int         c;
@@ -822,7 +822,7 @@ escape:
         if (diag && iscntrl( c) && ((char_type[ c] & SPA) == 0)
                 && (warn_level & 1))
             cwarn(
-            "Illegal control character %.0s0lx%02x in quotation"    /* _W1_ */
+            "Illegal control character %.0s0x%02lx in quotation"    /* _W1_ */
                     , NULL, (long) c, NULL);
         *out_p++ = c;
 chk_limit:
@@ -855,16 +855,16 @@ chk_limit:
                         goto  scan;         /* Splice the lines     */
                     /* Else end of file     */
                 } else {
-                    cerror( unterm_string, skip, 0L, NULL); /* _E_  */
+                    cerror( unterm_string, skip, 0L, NULL);         /* _E_  */
                 }
             } else if (delim == '\'') {
                 if (mcpp_mode != POST_STD && option_flags.lang_asm) {
                     /* STD, KR      */
                     if (warn_level & 1)
-                        cwarn( unterm_char, out, 0L, NULL); /* _W1_ */
+                        cwarn( unterm_char, NULL, 0L, NULL);        /* _W1_ */
                     goto  done;
                 } else {
-                    cerror( unterm_char, out, 0L, skip);    /* _E_  */
+                    cerror( unterm_char, skip, 0L, NULL);           /* _E_  */
                 }
             } else {
                 cerror( "Unterminated header name %s%.0ld%s"        /* _E_  */
@@ -875,9 +875,9 @@ chk_limit:
             if (mcpp_mode != POST_STD && option_flags.lang_asm) {
                 /* STD, KR      */
                 if (warn_level & 1)
-                    cwarn( empty_const, out, 0L, skip);     /* _W1_ */
+                    cwarn( empty_const, skip, 0L, NULL);            /* _W1_ */
             } else {
-                cerror( empty_const, out, 0L, skip);        /* _E_  */
+                cerror( empty_const, skip, 0L, NULL);               /* _E_  */
                 out_p = NULL;
                 goto  done;
             }
@@ -1561,9 +1561,9 @@ int     get_ch( void)
         if (! in_string && c == '\\' && *file->bptr == '\n'
                 && in_define        /* '\\''\n' is deleted in #define line, */
                     /*   provided the '\\' is not the 2nd byte of mbchar.   */
-                && ! last_is_mbchar( file->buffer, strlen( file->buffer) - 2
+                && ! last_is_mbchar( file->buffer, strlen( file->buffer) - 2)
                 && ! keep_spaces)
-            ) {
+        {
             if (*(file->bptr - 2) == ' ')
                 squeezews = TRUE;
         } else {
@@ -1587,13 +1587,13 @@ int     get_ch( void)
     free( file->buffer);                    /* Free buffer          */
     if (infile == NULL) {                   /* If at end of input   */
         free( file->filename);
-        free( file->src_dir);
+        free( (void *) file->src_dir);
         free( file);    /* full_fname is the same with filename for main file*/
         return  CHAR_EOF;                   /* Return end of file   */
     }
     if (file->fp) {                         /* Source file included */
         free( file->filename);              /* Free filename        */
-        free( file->src_dir);               /* Free src_dir         */
+        free( (void *) file->src_dir);      /* Free src_dir         */
         fclose( file->fp);                  /* Close finished file  */
         /* Do not free file->real_fname and file->full_fname        */
         cur_fullname = infile->full_fname;
@@ -1722,6 +1722,8 @@ com_start:
                     sp -= 2;
                     while (*sp != '\n')     /* Until end of line    */
                         mcpp_fputc( *sp++, OUT);
+                    mcpp_fputc( '\n', OUT);
+                    wrong_line = TRUE;
                 }
                 goto  end_line;
             default:                        /* Not a comment        */
@@ -1745,9 +1747,11 @@ not_comment:
                     *tp++ = '\t';
                 else
                     *tp++ = ' ';            /* Convert to ' '       */
-            } else if (! (char_type[ *(tp - 1) & UCHARMAX] & HSP)) {
+            } else if (temp == tp
+                       || ! (char_type[ *(tp - 1) & UCHARMAX] & HSP)) {
                 *tp++ = ' ';                /* Squeeze white spaces */
-            } else if (mcpp_mode == OLD_PREP && *(tp - 1) == COM_SEP) {
+            } else if (mcpp_mode == OLD_PREP
+                       && temp < tp && *(tp - 1) == COM_SEP) {
                 *(tp - 1) = ' ';    /* Replace COM_SEP with ' '     */
             }
             break;
@@ -1770,7 +1774,7 @@ not_comment:
         default:
             if (iscntrl( c)) {
                 cerror(             /* Skip the control character   */
-    "Illegal control character %.0s0x%lx, skipped the character"    /* _E_  */
+    "Illegal control character %.0s0x%02lx, skipped the character"  /* _E_  */
                         , NULL, (long) c, NULL);
             } else {                        /* Any valid character  */
                 *tp++ = c;
@@ -1944,6 +1948,11 @@ static char *   get_line(
             dump_string( NULL, ptr);
         }
         len = strlen( ptr);
+        if (len == 0) {
+                cerror( "Skipping line that begins with '\\0'", NULL, 0L, NULL);
+                continue;
+	}
+
         if (NBUFF - 1 <= ptr - infile->buffer + len
                 && *(ptr + len - 1) != '\n') {
                 /* The line does not yet end, though the buffer is full.    */
@@ -2297,8 +2306,8 @@ FILEINFO *  get_file(
         file->filename = NULL;
     }
     if (src_dir) {
-        file->src_dir = xmalloc( strlen( src_dir) + 1);
-        strcpy( file->src_dir, src_dir);
+        file->src_dir =
+            strcpy( xmalloc( strlen( src_dir) + 1), src_dir);
     } else {
         file->src_dir = NULL;
     }
